@@ -25,7 +25,7 @@ sascs_sasobjsp_host = 'vs2458.imb.ru'
 sascs_sasobjsp_port = '8591'
 sascs_workspace = 'DRKR - Workspace Server'
 sascs_sasapp_workspace = 'SASApp - Workspace Server'
-sascs_alloved_appservers = [sascs_sasapp_workspace]
+sascs_alloved_appservers = [sascs_workspace, sascs_sasapp_workspace]
 #**********************************
 sasvs235_sasobjsp_host = 'vs235.imb.ru'
 sasvs235_sasobjsp_port = '8591'
