@@ -6,6 +6,7 @@ import xml.etree.ElementTree as ET
 import saspy
 import logging
 import tempfile
+from datetime import datetime
 from ..globals import all_obj_types
 from sys import platform
 
@@ -19,6 +20,7 @@ else:
 
 
 def create_saspyconn(sasobjsp_host, sasobjsp_port, appserver, java=java):
+    logging.debug(f'Creating temporary sascfg file...')
     saspy_conn = tempfile.NamedTemporaryFile(mode='w+', delete=False)  # needed to manually deleted after, to be compitable with win
     saspy_conn.write(f"""SAS_config_names=['sasconnection']
 SAS_config_options = {{'lock_down': False,
@@ -35,6 +37,7 @@ sasconnection = {{'java'      : '{java}',
             }}""")
     saspy_conn.seek(0)
     saspy_conn.close()
+    logging.debug(f'Created temporary sascfg file.')
     return saspy_conn
 
 
@@ -63,6 +66,7 @@ class AbstractSAS(saspy.SASsession):
                                            cfgname='sasconnection',
                                            omruser=self.sasobjsp_user,
                                            omrpw=self.sasobjsp_pass)
+        logging.info(f'Pid in sas server side({self.sasobjsp_host}) is  {self.SASpid}')
         print(f'Pid in sas server side({self.sasobjsp_host}) is  {self.SASpid}')
 
     def read_dataset(self, tablename: str, libname: str = 'WORK'):
@@ -100,44 +104,29 @@ class AbstractSAS(saspy.SASsession):
         super(AbstractSAS, self).endsas()
 
 
-def get_sas_connect(sasobjsp_host, sasobjsp_port, sasobjsp_user, sasobjsp_pass, java=java):
-    """
-    LEGACY!!!!!
-    :param sasobjsp_host:
-    :param sasobjsp_port:
-    :param sasobjsp_user:
-    :param sasobjsp_pass:
-    :param java:
-    :return:
-    """
-    saspy_conn = tempfile.NamedTemporaryFile(mode='w+', delete=True)
-    saspy_conn.write(f"""SAS_config_names=['sasconnection']
-SAS_config_options = {{'lock_down': False,
-                    'verbose'  : True,
-                    'prompt'   : True
-                    }}
-SAS_output_options = {{'output' : 'html5'}} 
-sasconnection = {{'java'      : '{java}',
-            'iomhost'   : ['{sasobjsp_host}'],
-            'iomport'   : {sasobjsp_port},
-            'encoding': 'cyrillic',
-            'options' : ["-fullstimer"]
-            }}""")
-    saspy_conn.seek(0)
-    try:
-        sas = saspy.SASsession(cfgfile=saspy_conn.name, omruser=sasobjsp_user,
-                               omrpw=sasobjsp_pass)
-    except Exception as e:
-        print(f'error!!! {str(e)}')
-        return
-    return sas
+def get_obj_from_deploymap(root):
+    extracted_objects = []
+    meta_id = root.attrib.get('Id', root.text)
+    obj_name = root.attrib.get('Name', root.text)
+    obj_type = root.attrib.get('PublicType', root.text)
+    desc = root.attrib.get('Desc', root.text)
+    path = root.attrib.get('Path', root.text)
+    if meta_id and path:
+        meta_object = {'METADATA_ID': meta_id,
+                       'OBJECT_TYPE': obj_type,
+                       'OBJECT_NAME': obj_name,
+                       'OBJECT_PATH': path,
+                       'META_OBJ_DESC': desc}
+        extracted_objects.append(meta_object)
+    for elem in root.getchildren():
+        extracted_objects = extracted_objects + get_obj_from_deploymap(elem)
+    return extracted_objects
 
 
-def get_objects_from_spk(spk_path: str, log_path: str = None, object_types: list = None) -> dict:
+def get_objects_from_spk(spk_path, object_types=None):
     """
     Extracts information about metadata objects from spk
     :param spk_path: path (str) to spk
-    :param log_path: path(str) to log file
     :param object_types: list of objects types (str), needed to be extracted.
     If not provided - defaults to ['DeployedFlow', 'DeployedJob',
     'ExternalFile', 'Folder', 'Library', 'Role', 'Server', 'StoredProcess',
@@ -147,8 +136,11 @@ def get_objects_from_spk(spk_path: str, log_path: str = None, object_types: list
     list of path's --- FIX THAT DESCRIPTION!!!
     """
     logging.info(f"Object extraction from {spk_path} started.")
-    log_lines = []
-    success_status = False
+    errors = []
+    warnings = []
+    all_obj_types = ['DeployedFlow', 'DeployedJob', 'ExternalFile', 'Folder',
+                     'Library', 'Role', 'Server', 'StoredProcess', 'Table',
+                     'User', 'Job', 'PhysicalTable', 'Tree', 'SASLibrary']
     if not object_types:
         object_types = all_obj_types
     extracted_objects = {i: [] for i in object_types}
@@ -159,23 +151,21 @@ def get_objects_from_spk(spk_path: str, log_path: str = None, object_types: list
         if 'DeployMap.xml' in spk.namelist():
             deploymap = ET.parse(spk.open('DeployMap.xml'))
             root = deploymap.getroot()
-            for obj_type in extracted_objects.keys():
-                for item in root.findall(f'Objects/{obj_type}'):
-                    obj_path = item.get('Path').replace(f'({obj_type})', '')
-                    extracted_objects[obj_type].append(obj_path)
-                    logging.debug(f"Founded object {obj_type} in {obj_path}")
-                    log_lines += f"Extracted object {obj_type}: {obj_path}\n"
-            success_status = True
+            extracted_objects = get_obj_from_deploymap(root)
             logging.info(f"List of objects extracted successfully.")
+            spk.close()
+        else:
+            spk.close()
+            raise FileNotFoundError
     except Exception as e:
         logging.error(f"Errors during extraction: {str(e)}")
-        log_lines += str(e)
+        errors.append(str(e))
     finally:
-        logging.debug(f"Writing log to {log_path}")
-        if log_path:
-            with open(log_path, 'a') as logfile:
-                logfile.writelines(log_lines)
-        return {'Success': success_status, 'Objects': extracted_objects}
+        return {'Completed': True,
+                'Warnings': warnings,
+                'Errors': errors,
+                'Log': None,
+                'Objects': extracted_objects}
 
 
 def check_sas_code(sas_script_path: str = None,
@@ -235,9 +225,9 @@ def import_spk(sas_conn_file_path, spk_path, log_path, import_package_jar_path,
     """
     logging.info(f"Import of {spk_path} started.")
     logging.debug(f"'Check mode' is set to {safe_mode}")
-    execution_log_lines = []
-    execution_status = False
-    warnings = False
+    import_log_lines = []
+    warnings = []
+    errors = []
     command = ['-profile', sas_conn_file_path, '-package', spk_path, '-target',
                target, '-log', log_path, '-preservePaths', ]
     if platform == "linux" or platform == "linux2":
@@ -248,22 +238,32 @@ def import_spk(sas_conn_file_path, spk_path, log_path, import_package_jar_path,
         command.append('-noexecute')
     try:
         logging.debug(f"'Command to be run: {command}")
-        import_process = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        import_process = subprocess.run(command, stdout=subprocess.PIPE,
+                                        stderr=subprocess.PIPE)
         if import_process.returncode == 0:
-            execution_status = True
             logging.info(f"Import process finished successfully")
-        if import_process.returncode == 4:
-            execution_status = True
+        elif import_process.returncode == 4:
             logging.warning(f"Import process finished with Warnings")
-            warnings = True
+            warnings.append(f"Warning during SPK Import. Check log.")
+        else:
+            logging.error(f"Import process finished Errors")
+            errors.append(f"Import finished with errors. Check log.")
+        log = SPK_Import_Log(log_file_path=log_path)
+        log.parse()
+        warnings = log.warnings
+        errors = log.errors
     except Exception as e:
-        execution_log_lines += str(e)
+        import_log_lines += str(e)
         logging.error(f"Import process finished ERRORS! {str(e)}")
+        errors.append(f"Errors while performing import: {str(e)}")
     finally:
         logging.info(f"Log will be written to {log_path}")
-        with open(log_path, 'a') as logfile:
-            logfile.writelines(execution_log_lines)
-        return {'Success': execution_status, 'Warnings': warnings}
+        with open(log_path, 'a+') as logfile:
+            logfile.writelines(import_log_lines)
+        return {'Completed': True,
+                'Warnings': warnings,
+                'Errors': errors,
+                'Log': log_path}
 
 
 def check_import(sas_conn_file_path,
@@ -469,34 +469,43 @@ def backup_metadata(backup_script_path, comment, sas_conn_file_path, log_path):
     logging.info(f"Full metadata backup started")
     execution_log_lines = []
     env = os.environ.copy()
-    backup_success = False
     backup_name = None
-    warnings = False
+    warnings = []
+    errors = []
     command = [backup_script_path, '-comment', comment,
                '-profile', sas_conn_file_path, '-log', log_path]
     logging.debug(f"Command to run: {command}")
     try:
-        backup_process = subprocess.run(command, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        backup_process = subprocess.run(command, env=env,
+                                        stdout=subprocess.PIPE,
+                                        stderr=subprocess.PIPE)
+
         if backup_process.returncode == 0:
-            backup_success = True
-            backup_name = str(backup_process.stdout.readline()).split()[2]
+            backup_name = str(backup_process.stdout).split()[2]
             logging.info(f"Backup {backup_name} completed successfully.")
         elif backup_process.returncode == 4:
-            backup_success = False
-            backup_name = str(backup_process.stdout.readline()).split()[2]
-            logging.warning(f"Backup {backup_name} completed with WARNINGS.")
-            warnings = True
+            logging.warning(f"Warnings during backup.")
+            warnings.append(f"There is a warning...")
         else:
-            # execution_log_lines += backup_process.stderr
-            logging.warning(f"Backup NOT completed!")
+            logging.error(f"Backup NOT completed!")
+            errors.append(f"Errors during backup.")
+        execution_log_lines += f"{80 * '='}\n"
+        execution_log_lines += f"Captured output (May contain more data...)\n"
+        execution_log_lines += f"{80 * '='}\n"
+        execution_log_lines += str(backup_process.stdout)
     except Exception as e:
-        execution_log_lines += str(e)
         logging.error(f"Errors during backup process! {str(e)}")
+        execution_log_lines += str(e)
+        errors.append(f"Errors while performing backup.")
     finally:
         logging.info(f"Log will be written to {log_path}")
         with open(log_path, 'a+') as log:
             log.writelines(execution_log_lines)
-        return {'Success': backup_success, 'Backup_name': backup_name}
+        return {'Completed': True,
+                'Warnings': warnings,
+                'Errors': errors,
+                'Log': log_path,
+                'Backup_name': backup_name}
 
 
 def recover_metadata(recover_script_path, backup_name, comment,
@@ -514,30 +523,160 @@ def recover_metadata(recover_script_path, backup_name, comment,
     logging.info(f"Metadata recovery from backup {backup_name} started.")
     execution_log_lines = []
     env = os.environ.copy()
-    restore_success = False
-    warnings = False
+    warnings = []
+    errors = []
     command = [recover_script_path, backup_name, '-comment', comment,
                '-profile', sas_conn_file_path, '-log', log_path]
     logging.info(f"Command to run: {command}")
     try:
-        backup_process = subprocess.run(command, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        print(80 * '*')
+        backup_process = subprocess.run(command, env=env,
+                                        stdout=subprocess.PIPE,
+                                        stderr=subprocess.PIPE)
+        execution_log_lines += f"{80 * '='}\n"
+        execution_log_lines += f"Captured output (May contain more data...)\n"
+        execution_log_lines += f"{80 * '='}\n"
+        execution_log_lines += str(backup_process.stdout)
         if backup_process.returncode == 0:
-            restore_success = True
             logging.info(f"Restore from backup {backup_name} finished successfully")
         elif backup_process.returncode == 4:
-            restore_success = False
-            warnings = True
+            warnings.append(f"There is a warning.")
             logging.warning(f"Restore from backup {backup_name} finished with warnings")
-            # execution_log_lines += redeploy_process.stdout
         else:
-            logging.warning(f"Restore from backup {backup_name} not finished!")
-            # execution_log_lines += redeploy_process.stderr
+            restore_success = False
+            logging.error(f"Restore from backup {backup_name} not finished!")
+            errors.append(f"Restore from backup {backup_name} not finished!")
     except Exception as e:
+        restore_success = False
         execution_log_lines += str(e)
         logging.error(f"Restore from backup {backup_name} not finished. ERRORS: {str(e)}")
+        errors.append(f"Restore from backup {backup_name} not finished. ERRORS: {str(e)}")
     finally:
         logging.info(f"Log file will be written to {log_path}")
-        with open(log_path, 'a') as log:
+        with open(log_path, 'a+') as log:
             log.writelines(execution_log_lines)
-        return {'Success': restore_success}
+        return {'Completed': True,
+                'Warnings': warnings,
+                'Errors': errors,
+                'Log': log_path}
+
+
+def export_object_relations(report_process_file, sas_relationship_reporter_path, object, user, host, password, port='7980'):
+    logging.info(f"Object {object} relations export started.")
+    reports = []
+    warnings = []
+    errors = []
+    env = os.environ.copy()
+    report_types = ['lineage', 'impact', 'directDependencies', 'indirectDependencies']  # this should be cofigurable!!!!
+    for report in report_types:
+        logging.info(f"Generating {report} report.")
+        command = [sas_relationship_reporter_path, '-report', report, object, '-user', user,
+                '-host', host, '-password', password, '-port', port, '-format', 'CSV']
+        logging.debug(f"Command to be launched: {command}")
+        report_process = subprocess.run(command, env=env,
+                                            stdout=subprocess.PIPE,
+                                            stderr=subprocess.PIPE)
+        report_process_file_name = f"{object.split('/')[-1]}_{report}_report_{datetime.now().strftime(r'%Y_%m_%d-%H:%M:%S')}.csv"
+        # report_process_file = os.path.join(config['SETTINGS']['storage'], report_process_file_name)
+        with open(report_process_file, 'wb') as s:
+            s.write(report_process.stdout)
+        logging.info(f"Report written to file {report_process_file}")
+        reports.append(report_process_file)
+        if report_process.returncode == 0:
+            logging.info(f"Loaded {report} report about {object}.")
+        elif report_process.returncode == 4:
+            warnings.append(f'Warnings during genetating {report} report.')
+            logging.warning(f"Warnings during {report} report about {object}")
+        else:
+            errors.append(f'Errors during genetating {report} report.')
+            logging.error(f"Errors during {report} report about {object}")
+    return {'Completed': True,
+            'Reports': reports,
+            'Warnings': warnings,
+            'Errors': errors,
+            'Log': None}
+
+
+def load_object_report(loader_process_log ,sas_relationship_loader_path, object, user, host, password, port='7980'):
+    logging.info(f"Relationship analyze for {object} started.")
+    env = os.environ.copy()
+    warnings = []
+    errors = []
+    command = [sas_relationship_loader_path, object, '-user', user,
+               '-host', host, '-password', password, '-port', port]
+    logging.debug(f"Command to be executed: {command}")
+    loader_process = subprocess.run(command, env=env,
+                                        stdout=subprocess.PIPE,
+                                        stderr=subprocess.PIPE)
+    loader_process_log_name = f"{object.split('/')[-1]}_info_load_{datetime.now().strftime(r'%Y_%m_%d-%H:%M:%S')}.log"
+    # loader_process_log = os.path.join(config['SETTINGS']['storage'], loader_process_log_name)
+    with open(loader_process_log, 'wb') as s:
+        s.write(loader_process.stdout)
+    logging.debug(f"Log was written to {loader_process_log}")
+    if loader_process.returncode == 0:
+        logging.info(f"Loaded info about {object}.")
+    elif loader_process.returncode == 4:
+        warnings.append(f"Warnings during loading info about {object}")
+        logging.warning(f"Warnings during loading info about {object}")
+    else:
+        errors.append(f"Errors during loading info about {object}")
+        logging.error(f"Errors during loading info about {object}")
+    return {'Completed': True,
+            'Warnings': warnings,
+            'Errors': errors,
+            'Log': loader_process_log}
+
+
+def get_sas_objects(sas_list_objects_path, connection_file, start_folfer='/', sort='path'):
+    extracted_objects = []
+    warnings = []
+    errors = []
+    objects_xml = tempfile.NamedTemporaryFile(mode='w+', delete=True)
+    env = os.environ.copy()
+    command = [sas_list_objects_path, '-folderTree', start_folfer, '-sort', sort,
+               '-profile', connection_file, '-format', 'xml']
+    logging.debug(f"Running {command}")
+    export_process = subprocess.run(command, env=env,
+                                        stdout=subprocess.PIPE,
+                                        stderr=subprocess.PIPE)
+    with open(objects_xml.name, 'wb') as s:
+        s.write(export_process.stdout)
+    mapping = ET.parse(objects_xml.name)
+    root = mapping.getroot()
+    format = '%Y-%m-%dT%H:%M:%SZ'  # move to config???
+    for tag in root.findall('Object'):
+        metapath = tag.get('Path', '')
+        name = tag.get('Name', '')
+        type = tag.get('Type', '')
+        typelabel = tag.get('TypeLabel', '')
+        description = tag.get('Description', '')
+        creation_dttm = tag.get('Created', '')
+        if creation_dttm:
+            try:
+                creation_dttm = datetime.strptime(creation_dttm, format)
+            except ValueError as error:
+                creation_dttm = ''
+        modify_dttm = tag.get('Modified', '')
+        if modify_dttm:
+            try:
+                modify_dttm = datetime.strptime(modify_dttm, format)
+            except ValueError as error:
+                modify_dttm = ''
+        created_by = tag.get('CreatedBy', '')
+        modified_by = tag.get('ModifiedBy', '')
+        meta_object = {'METADATA_ID': '',
+                        'OBJECT_TYPE': type,
+                        'OBJECT_NAME': name,
+                        'OBJECT_PATH': metapath,
+                        'META_OBJ_DESC': description,
+                        'CREATION_DTTM':creation_dttm,
+                        'MODIFY_DTTM':modify_dttm,
+                        'CREATED_BY':created_by,
+                        'MODIFIED_BY':modified_by,
+                        'TYPELABEL':typelabel}
+        print(f"{meta_object}")
+        extracted_objects.append(meta_object)
+    return {'Completed': True,
+            'Warnings': warnings,
+            'Errors': errors,
+            'Log': None,
+            'Objects': extracted_objects}
