@@ -172,4 +172,107 @@ proc sql;
 	delete from Libraries_final
 	where libname ='';
 quit;
-""")
\ No newline at end of file
+""")
+
+extract_authdomains_template = SasCodeTemplate("""
+
+proc sql;
+create table authdoms
+(authdom char(50)
+,userid char(50)
+,pwencode char(200)
+,pwdecode char(200)
+,desc char(500)
+/*,system char(50)*/);
+quit;
+
+
+%macro Auth(Authdom,UserId,Password,Desc);
+	
+	proc groovy;
+		add sasjar="sas.core";
+		eval "import com.sas.util.SasPasswordString; exports.OpenPassword=SasPasswordString.decode(""&Password"")";
+	run;
+
+	proc sql inobs=1;
+		insert into authdoms
+		select "&Authdom.","&UserId.","&Password.","&OpenPassword.","&Desc."
+		from sashelp.table;
+	quit;
+	%let OpenPassword=;
+%mend;
+
+%let metasrv="sasmeta.dwhgridprod.imb.ru";
+%let metaprt=8561;
+/*%let metausr="sasadm@saspw";*/
+%let metausr="mb30728";
+/*%let metapwd="{SAS002}FACB520F2A1764C93DD09B70191005080290E78A";*/
+%let metapwd="{SAS002}";
+%let metarep="Foundation";
+
+options 
+		metaserver=&metasrv
+        metaport=&metaprt
+        metauser=&metausr
+        metapass=&metapwd
+        metarepository=&metarep;
+
+/*************************************************************/
+  data metadata_authdomains;
+  length uri authdom desc outbound loginuri userid password AuthId openpassword$256 ;
+  keep authdom desc outbound userid password openpassword;
+  nobj=0;
+  n=1;
+  uri='';
+         /***Determine how many authdom there are***/
+  nobj=metadata_getnobj("omsobj:AuthenticationDomain?@Id contains '.'",n,uri);
+         /***Retrieve the attributes for all libraries, if there are any***/
+  if n>0 then do n=1 to nobj;
+	AuthId='';
+	authdom='';
+	desc='';
+	outbound='';
+	loginuri='';
+	UserID='';
+	Password='';
+
+    nobj=metadata_getnobj("omsobj:AuthenticationDomain?@Id contains '.'",n,uri);
+	rc= metadata_getattr(uri,"Id",AuthId);
+    rc= metadata_getattr(uri, "Name", authdom);
+    rc= metadata_getattr(uri, "Desc", desc);
+    rc= metadata_getattr(uri, "OutboundOnly", outbound);
+
+	rc=1;
+    tn=1;
+	nobj2=0;
+
+	nobj2=metadata_getnasn("OMSOBJ:AuthenticationDomain\"!!AuthId,"Logins",tn,loginuri);
+	if nobj2<0 then do; output; end;
+	do tn=1 to nobj2;
+ 	   rc=metadata_getnasn("OMSOBJ:AuthenticationDomain\"!!AuthId,"Logins",tn,loginuri);
+  	   if rc>0 then do;
+   	     rc2=metadata_getattr(loginuri,"UserID",UserID);
+    	 rc2=metadata_getattr(loginuri,"Password",Password);
+		 output;
+		 UserID='';
+		 Password=''; 
+       end;
+    end;
+  end;
+ run;
+
+
+data run;
+	set metadata_authdomains;
+	str_macro = catt('%Auth(', authdom,',', userid,',',password,',',desc,');');
+	call execute(str_macro);
+run;
+
+
+data _null_;
+	set run;
+	call execute(str_macro);
+run;
+
+                                                """)
+
